<?php

/**
 * Database operations class.
 */
class DB {

    const DB_NAME  = 'sql_injection';
    const PORT     = 5432;            // PostgreSQL default port.
    const DB_HOST  = 'localhost';
    const USERNAME = 'sql_injection_user';
    const PASSWORD = 'sql_injection';
    const SUSPICIOUS_LOGFILE = 'suspicious_activity.log';

    /**
     * Forbidden input tokens.
     */
    private $blacklist = "/(\s*union\s*|where\s*'1')/";

    /**
     * Database connection object.
     */
    private $connection;

    /**
     * Prepared statement for login.
     */
    private $authenticationStatement;

    /**
     * Connects to a database with the defined class constants, constructing a connection string like
     * "host='db_host' port='port' dbname='dbname' user='user' password='password'".
     *
     * @return True if the connection was made, false if it was not.
     */
    public function connect() {
        $connectionString = 'host=' . self::DB_HOST . ' port=' . self::PORT . ' dbname=' . self::DB_NAME
            . ' user=' . self::USERNAME . ' password=' . self::PASSWORD;

        $connection = pg_connect($connectionString);

        if ($connection) {
            $connected = true;
            $this->connection = $connection;
            $this->createAuthenticationStatement();
        } else {
            $connected = false;
        }

        return $connected;
    }

    /**
     * Closes the opened connection to the database.
     *
     * @return True if the connection was closed succesfully; false if an error happened.
     */
    public function disconnect() {
        $disconnected = pg_close($this->connection);

        return $disconnected;
    }

    /**
     * Creates the prepared statement for user authentication.
     */
    protected function createAuthenticationStatement() {
        $sql = "SELECT username
                FROM   Users
                WHERE  username = $1
                   AND password = $2
                ";

        $this->authenticationStatement = pg_prepare($this->connection, 'authenticate', $sql);
    }

    /**
     * Authenticates an user with its password, looking if exists a row for the given credentials.
     *
     * @param $username The user to authenticate.
     * @param $password The password to authenticate the user.
     * @return User name if the user was authenticated, false if not.
     */
    public function authenticateUser($username, $password) {
        if (!$this->blacklistPositive([$username, $password])) {
            $result = pg_execute($this->connection, 'authenticate', [$username, $password]);
            $row = pg_fetch_row($result);

            if (!$row) {
                $authenticated = false;
            } else {
                $authenticated = $row[0];
            }
        } else {
            $authenticated = false;
        }

        return $authenticated;
    }

    /**
     * Checks provided inputs to look for patterns defined in class property's black list array.
     *
     * @param $userInputs Inputs to check.
     * @return True if a positive is found, false if not.
     */
    protected function blacklistPositive($userInputs) {
        foreach ($userInputs as $userInput) {
            $positive = preg_match($this->blacklist, $userInput);

            if ($positive) {
                $this->logBlacklistPositive($userInput);
                return true;
            }
        }

        return false;
    }

    /**
     * Creates a record when blaclistPositive() function returns true.
     * Datetime, remote IP and the input that triggered the action are saved.
     */
    protected function logBlacklistPositive($trigger) {
        $datetime = date('d-m-Y h:i:s');
        $ip = $_SERVER['REMOTE_ADDR'];

        file_put_contents(self::SUSPICIOUS_LOGFILE, "$datetime; $ip; $trigger\n", FILE_APPEND);
    }
}
